/*
 * gsttiimgdec1.c
 *
 * This file defines the "TIImgdec1" element, which decodes a JPEG image using
 * the xDM 1.x interface.
 *
 * Example usage:
 *     gst-launch filesrc location=<input file> ! TIImgdec1 ! filesink location=<output file>
 *
 * Original Author:
 *     Chase Maupin, Texas Instruments, Inc.
 *
 * Copyright (C) 2008-2010 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation version 2.1 of the License.
 *
 * This program is distributed #as is# WITHOUT ANY WARRANTY of any kind,
 * whether express or implied; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <gst/gst.h>
#include <gst/video/video.h>
#include <ctype.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Cpu.h>
#include <ti/sdo/dmai/Buffer.h>
#include <ti/sdo/dmai/BufferGfx.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/ce/Idec1.h>

#include "gsttiimgdec1.h"
#include "gsttidmaibuffertransport.h"
#include "gstticodecs.h"
#include "gsttithreadprops.h"
#include "gstticommonutils.h"

/* Declare variable used to categorize GST_LOG output */
GST_DEBUG_CATEGORY_STATIC (gst_tiimgdec1_debug);
#define GST_CAT_DEFAULT gst_tiimgdec1_debug

/* Element property identifiers */
enum
{
  PROP_0,
  PROP_ENGINE_NAME,     /* engineName     (string)  */
  PROP_CODEC_NAME,      /* codecName      (string)  */
  PROP_NUM_OUTPUT_BUFS, /* numOutputBufs  (int)     */
  PROP_FRAMERATE,       /* frameRate      (int)     */
  PROP_RESOLUTION,      /* resolution     (string)  */
  PROP_DISPLAY_BUFFER,  /* displayBuffer  (boolean) */
  PROP_GEN_TIMESTAMPS   /* genTimeStamps  (boolean) */
};

/* Define sink (input) pad capabilities */
static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE(
    "sink",
    GST_PAD_SINK,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS
    ("video/x-jpeg, "
        "width=(int)[ 1, MAX ], "
        "height=(int)[ 1, MAX ], "
        "framerate=(fraction)[ 0, MAX ]"
    )
);

/* Define source (output) pad capabilities */
static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE(
    "src",
    GST_PAD_SRC,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS
    ("video/x-raw-yuv, "                        /* UYVY */
         "format=(fourcc)UYVY, "
         "framerate=(fraction)[ 0, MAX ], "
         "width=(int)[ 1, MAX ], "
         "height=(int)[ 1, MAX ]"
    )
);

/* Declare a global pointer to our element base class */
static GstElementClass *parent_class = NULL;

/* Static Function Declarations */
static void
 gst_tiimgdec1_base_init(gpointer g_class);
static void
 gst_tiimgdec1_class_init(GstTIImgdec1Class *g_class);
static void
 gst_tiimgdec1_init(GstTIImgdec1 *object, GstTIImgdec1Class *g_class);
static void
 gst_tiimgdec1_set_property (GObject *object, guint prop_id,
     const GValue *value, GParamSpec *pspec);
static void
 gst_tiimgdec1_get_property (GObject *object, guint prop_id, GValue *value,
     GParamSpec *pspec);
static gboolean
 gst_tiimgdec1_set_sink_caps(GstPad *pad, GstCaps *caps);
static gboolean
 gst_tiimgdec1_set_source_caps(GstTIImgdec1 *imgdec1, Buffer_Handle hBuf);
static gboolean
 gst_tiimgdec1_sink_event(GstPad *pad, GstEvent *event);
static GstFlowReturn
 gst_tiimgdec1_chain(GstPad *pad, GstBuffer *buf);
static gboolean
 gst_tiimgdec1_init_image(GstTIImgdec1 *imgdec1);
static gboolean
 gst_tiimgdec1_exit_image(GstTIImgdec1 *imgdec1);
static GstStateChangeReturn
 gst_tiimgdec1_change_state(GstElement *element, GstStateChange transition);
static void*
 gst_tiimgdec1_decode_thread(void *arg);
static void
 gst_tiimgdec1_drain_pipeline(GstTIImgdec1 *imgdec1);
static GstClockTime
 gst_tiimgdec1_frame_duration(GstTIImgdec1 *imgdec1);
static char *
 gst_tiimgdec1_codec_color_space_to_str(int cspace);
static int 
 gst_tiimgdec1_codec_color_space_to_fourcc(int cspace);
static gboolean 
    gst_tiimgdec1_codec_start (GstTIImgdec1  *imgdec1);
static gboolean 
    gst_tiimgdec1_codec_stop (GstTIImgdec1  *imgdec1);
static void 
    gst_tiimgdec1_init_env(GstTIImgdec1 *imgdec1);

/******************************************************************************
 * gst_tiimgdec1_class_init_trampoline
 *    Boiler-plate function auto-generated by "make_element" script.
 ******************************************************************************/
static void gst_tiimgdec1_class_init_trampoline(gpointer g_class,
                gpointer data)
{
    GST_LOG("Begin\n");
    parent_class = (GstElementClass*) g_type_class_peek_parent(g_class);
    gst_tiimgdec1_class_init((GstTIImgdec1Class*)g_class);
    GST_LOG("Finish\n");
}


/******************************************************************************
 * gst_tiimgdec1_get_type
 *    Boiler-plate function auto-generated by "make_element" script.
 *    Defines function pointers for initialization routines for this element.
 ******************************************************************************/
GType gst_tiimgdec1_get_type(void)
{
    static GType object_type = 0;

    if (G_UNLIKELY(object_type == 0)) {
        static const GTypeInfo object_info = {
            sizeof(GstTIImgdec1Class),
            gst_tiimgdec1_base_init,
            NULL,
            gst_tiimgdec1_class_init_trampoline,
            NULL,
            NULL,
            sizeof(GstTIImgdec1),
            0,
            (GInstanceInitFunc) gst_tiimgdec1_init
        };

        object_type = g_type_register_static((gst_element_get_type()),
                          "GstTIImgdec1", &object_info, (GTypeFlags)0);

        /* Initialize GST_LOG for this object */
        GST_DEBUG_CATEGORY_INIT(gst_tiimgdec1_debug, "TIImgdec1", 0,
            "TI xDM 1.0 Image Decoder");

        GST_LOG("initialized get_type\n");
    }

    return object_type;
};


/******************************************************************************
 * gst_tiimgdec1_base_init
 *    Boiler-plate function auto-generated by "make_element" script.
 *    Initializes element base class.
 ******************************************************************************/
static void gst_tiimgdec1_base_init(gpointer gclass)
{
    static GstElementDetails element_details = {
        "TI xDM 1.0 Image Decoder",
        "Codec/Decoder/Image",
        "Decodes a JPEG using an xDM 1.0-based codec",
        "Chase Maupin; Texas Instruments, Inc."
    };
    GST_LOG("Begin\n");

    GstElementClass *element_class = GST_ELEMENT_CLASS(gclass);

    gst_element_class_add_pad_template(element_class,
        gst_static_pad_template_get (&src_factory));
    gst_element_class_add_pad_template(element_class,
        gst_static_pad_template_get (&sink_factory));
    gst_element_class_set_details(element_class, &element_details);
    GST_LOG("Finish\n");
}


/******************************************************************************
 * gst_tiimgdec1_class_init
 *    Boiler-plate function auto-generated by "make_element" script.
 *    Initializes the TIImgdec1 class.
 ******************************************************************************/
static void gst_tiimgdec1_class_init(GstTIImgdec1Class *klass)
{
    GObjectClass    *gobject_class;
    GstElementClass *gstelement_class;

    gobject_class    = (GObjectClass*)    klass;
    gstelement_class = (GstElementClass*) klass;

    GST_LOG("Begin\n");

    gobject_class->set_property = gst_tiimgdec1_set_property;
    gobject_class->get_property = gst_tiimgdec1_get_property;

    gstelement_class->change_state = gst_tiimgdec1_change_state;

    g_object_class_install_property(gobject_class, PROP_ENGINE_NAME,
        g_param_spec_string("engineName", "Engine Name",
            "Engine name used by Codec Engine", "unspecified",
            G_PARAM_READWRITE));

    g_object_class_install_property(gobject_class, PROP_CODEC_NAME,
        g_param_spec_string("codecName", "Codec Name", "Name of image codec",
            "unspecified", G_PARAM_READWRITE));

    g_object_class_install_property(gobject_class, PROP_RESOLUTION,
        g_param_spec_string("resolution", "Image Resolution",
            "The resolution of the input image ('width'x'height')\n",
            "720x480", G_PARAM_WRITABLE));

    /* We allow more that one output buffer because this is the buffer that
     * is sent to the downstream element.  It may be that we need to have
     * more than 1 buffer for the mjpeg case if the downstream element
     * doesn't give the buffer back in time for the codec to use.
     */
    g_object_class_install_property(gobject_class, PROP_NUM_OUTPUT_BUFS,
        g_param_spec_int("numOutputBufs",
            "Number of Ouput Buffers",
            "Number of output buffers to allocate for codec",
            1, G_MAXINT32, 1, G_PARAM_WRITABLE));

    /* For MJPEG we will communicate a frame rate to the down stream
     * element.  This can be ignored normally.
     */
    g_object_class_install_property(gobject_class, PROP_FRAMERATE,
        g_param_spec_int("frameRate",
            "Frame rate to play output",
            "Communicate this framerate to downstream elements.  The frame "
            "rate specified should be an integer.  If 29.97fps is desired, "
            "specify 30 for this setting",
            1, G_MAXINT32, 30, G_PARAM_WRITABLE));

    g_object_class_install_property(gobject_class, PROP_DISPLAY_BUFFER,
        g_param_spec_boolean("displayBuffer", "Display Buffer",
            "Display circular buffer status while processing",
            FALSE, G_PARAM_WRITABLE));

    g_object_class_install_property(gobject_class, PROP_GEN_TIMESTAMPS,
        g_param_spec_boolean("genTimeStamps", "Generate Time Stamps",
            "Set timestamps on output buffers",
            TRUE, G_PARAM_WRITABLE));

    GST_LOG("Finish\n");
}

/*****************************************************************************
 * gst_tiimgdec1_init_env
 *  Initialize element property default by reading environment variables.
 *****************************************************************************/
static void gst_tiimgdec1_init_env(GstTIImgdec1 *imgdec1)
{
    GST_LOG("gst_tiimgdec1_init_env - begin");

    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_engineName")) {
        imgdec1->engineName = gst_ti_env_get_string("GST_TI_TIImgdec1_engineName");
        GST_LOG("Setting engineName=%s\n", imgdec1->engineName);
    }

    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_codecName")) {
        imgdec1->codecName = gst_ti_env_get_string("GST_TI_TIImgdec1_codecName");
        GST_LOG("Setting codecName=%s\n", imgdec1->codecName);
    }
    
    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_numOutputBufs")) {
        imgdec1->numOutputBufs = 
                            gst_ti_env_get_int("GST_TI_TIImgdec1_numOutputBufs");
        GST_LOG("Setting numOutputBufs=%ld\n", imgdec1->numOutputBufs);
    }

    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_displayBuffer")) {
        imgdec1->displayBuffer = 
                gst_ti_env_get_boolean("GST_TI_TIImgdec1_displayBuffer");
        GST_LOG("Setting displayBuffer=%s\n",
                 imgdec1->displayBuffer  ? "TRUE" : "FALSE");
    }
 
    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_genTimeStamps")) {
        imgdec1->genTimeStamps = 
                gst_ti_env_get_boolean("GST_TI_TIImgdec1_genTimeStamps");
        GST_LOG("Setting genTimeStamps =%s\n", 
                    imgdec1->genTimeStamps ? "TRUE" : "FALSE");
    }
    
    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_framerate")) {
        imgdec1->framerateNum = gst_ti_env_get_int("GST_TI_TIImgdec1_framerate");
        imgdec1->framerateDen = 1;
        
        /* If 30fps was specified, use 29.97 */        
        if (imgdec1->framerateNum == 30) {
            imgdec1->framerateNum = 30000;
            imgdec1->framerateDen = 1001;
        }
    }

    if (gst_ti_env_is_defined("GST_TI_TIImgdec1_resolution")) {
        sscanf(gst_ti_env_get_string("GST_TI_TIImgdec1_resolution"), "%dx%d", 
                &imgdec1->width,&imgdec1->height);
        GST_LOG("Setting resolution=%dx%d\n", imgdec1->width, imgdec1->height);
    }
    
    GST_LOG("gst_tiimgdec_init_env - end");
}

/******************************************************************************
 * gst_tiimgdec1_init
 *    Initializes a new element instance, instantiates pads and sets the pad
 *    callback functions.
 ******************************************************************************/
static void gst_tiimgdec1_init(GstTIImgdec1 *imgdec1, GstTIImgdec1Class *gclass)
{
    GST_LOG("Begin\n");
    /* Instantiate decoded image sink pad
     *
     * Fixate on our static template caps insteam of writing a getcaps
     * function which is overkill for this element.
     */
    imgdec1->sinkpad =
        gst_pad_new_from_static_template(&sink_factory, "sink");
    gst_pad_set_setcaps_function(
        imgdec1->sinkpad, GST_DEBUG_FUNCPTR(gst_tiimgdec1_set_sink_caps));
//    gst_pad_set_getcaps_function(
//        imgdec1->sinkpad, GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));
    gst_pad_set_event_function(
        imgdec1->sinkpad, GST_DEBUG_FUNCPTR(gst_tiimgdec1_sink_event));
    gst_pad_set_chain_function(
        imgdec1->sinkpad, GST_DEBUG_FUNCPTR(gst_tiimgdec1_chain));
    gst_pad_fixate_caps(imgdec1->sinkpad,
        gst_caps_make_writable(
            gst_caps_copy(gst_pad_get_pad_template_caps(imgdec1->sinkpad))));

    /* Instantiate deceoded image source pad
     *
     * Fixate on our static template caps insteam of writing a getcaps
     * function which is overkill for this element.
     */
    imgdec1->srcpad =
        gst_pad_new_from_static_template(&src_factory, "src");
//    gst_pad_set_getcaps_function(
//        imgdec1->srcpad, GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));
    gst_pad_fixate_caps(imgdec1->srcpad,
        gst_caps_make_writable(
            gst_caps_copy(gst_pad_get_pad_template_caps(imgdec1->srcpad))));

    /* Add pads to TIImgdec1 element */
    gst_element_add_pad(GST_ELEMENT(imgdec1), imgdec1->sinkpad);
    gst_element_add_pad(GST_ELEMENT(imgdec1), imgdec1->srcpad);

    /* Initialize TIImgdec1 state */
    imgdec1->engineName         = NULL;
    imgdec1->codecName          = NULL;
    imgdec1->displayBuffer      = FALSE;
    imgdec1->genTimeStamps      = FALSE;
    imgdec1->width              = 0;
    imgdec1->height             = 0;

    imgdec1->hEngine            = NULL;
    imgdec1->hIe                = NULL;
    imgdec1->drainingEOS        = FALSE;
    imgdec1->threadStatus       = 0UL;
    imgdec1->capsSet            = FALSE;

    imgdec1->waitOnDecodeThread = NULL;
    imgdec1->waitOnDecodeDrain  = NULL;

    imgdec1->framerateNum       = 0;
    imgdec1->framerateDen       = 0;

    imgdec1->numOutputBufs      = 0UL;
    imgdec1->hOutBufTab         = NULL;
    imgdec1->circBuf            = NULL;

    gst_tiimgdec1_init_env(imgdec1);

    GST_LOG("Finish\n");
}

/******************************************************************************
 * gst_tiimgdec1_set_property
 *     Set element properties when requested.
 ******************************************************************************/
static void gst_tiimgdec1_set_property(GObject *object, guint prop_id,
                const GValue *value, GParamSpec *pspec)
{
    GstTIImgdec1 *imgdec1 = GST_TIIMGDEC1(object);

    GST_LOG("Begin\n");

    switch (prop_id) {
        case PROP_ENGINE_NAME:
            if (imgdec1->engineName) {
                g_free((gpointer)imgdec1->engineName);
            }
            imgdec1->engineName =
                (gchar*)g_malloc(strlen(g_value_get_string(value)) + 1);
            strcpy((gchar *)imgdec1->engineName, g_value_get_string(value));
            GST_LOG("setting \"engineName\" to \"%s\"\n", imgdec1->engineName);
            break;
        case PROP_CODEC_NAME:
            if (imgdec1->codecName) {
                g_free((gpointer)imgdec1->codecName);
            }
            imgdec1->codecName =
                (gchar*)g_malloc(strlen(g_value_get_string(value)) + 1);
            strcpy((gchar*)imgdec1->codecName, g_value_get_string(value));
            GST_LOG("setting \"codecName\" to \"%s\"\n", imgdec1->codecName);
            break;
        case PROP_RESOLUTION:
            sscanf(g_value_get_string(value), "%dx%d", &imgdec1->width,
                    &imgdec1->height);
            GST_LOG("setting \"resolution\" to \"%dx%d\"\n",
                imgdec1->width, imgdec1->height);
            break;
        case PROP_NUM_OUTPUT_BUFS:
            imgdec1->numOutputBufs = g_value_get_int(value);
            GST_LOG("setting \"numOutputBufs\" to \"%ld\"\n",
                imgdec1->numOutputBufs);
            break;
        case PROP_FRAMERATE:
        {
            imgdec1->framerateNum = g_value_get_int(value);
            imgdec1->framerateDen = 1;

            /* If 30fps was specified, use 29.97 */
            if (imgdec1->framerateNum == 30) {
                imgdec1->framerateNum = 30000;
                imgdec1->framerateDen = 1001;
            }

            GST_LOG("setting \"frameRate\" to \"%2.2lf\"\n",
                (gdouble)imgdec1->framerateNum /
                (gdouble)imgdec1->framerateDen);
            break;
        }
        case PROP_DISPLAY_BUFFER:
            imgdec1->displayBuffer = g_value_get_boolean(value);
            GST_LOG("setting \"displayBuffer\" to \"%s\"\n",
                imgdec1->displayBuffer ? "TRUE" : "FALSE");
            break;
        case PROP_GEN_TIMESTAMPS:
            imgdec1->genTimeStamps = g_value_get_boolean(value);
            GST_LOG("setting \"genTimeStamps\" to \"%s\"\n",
                imgdec1->genTimeStamps ? "TRUE" : "FALSE");
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
            break;
    }

    GST_LOG("Finish\n");
}

/******************************************************************************
 * gst_tiimgdec1_get_property
 *     Return values for requested element property.
 ******************************************************************************/
static void gst_tiimgdec1_get_property(GObject *object, guint prop_id,
                GValue *value, GParamSpec *pspec)
{
    GstTIImgdec1 *imgdec1 = GST_TIIMGDEC1(object);

    GST_LOG("Begin\n");

    switch (prop_id) {
        case PROP_ENGINE_NAME:
            g_value_set_string(value, imgdec1->engineName);
            break;
        case PROP_CODEC_NAME:
            g_value_set_string(value, imgdec1->codecName);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
            break;
    }

    GST_LOG("Finish\n");
}

/*******************************************************************************
 * gst_tiimgdec1_set_sink_caps_helper
 *     This function will look at the capabilities given and set the values
 *     for the decoder if they were not specified on the command line.
 *     It returns TRUE if everything passes and FALSE if there is no
 *     capability in the buffer and the value was not specified on the
 *     command line.
 ******************************************************************************/
static gboolean gst_tiimgdec1_set_sink_caps_helper(GstPad *pad, GstCaps *caps)
{
    GstTIImgdec1 *imgdec1;
    GstStructure *capStruct;
    const gchar  *mime;
    GstTICodec   *codec = NULL;

    GST_LOG("Begin\n");

    imgdec1   = GST_TIIMGDEC1(gst_pad_get_parent(pad));

    /* Set the default values */
    imgdec1->params = Idec1_Params_DEFAULT;
    imgdec1->dynParams = Idec1_DynamicParams_DEFAULT;

    /* Set codec to JPEG Decoder */
    codec = gst_ticodec_get_codec("JPEG Image Decoder");

    /* Report if the required codec was not found */
    if (!codec) {
        GST_ELEMENT_ERROR(imgdec1, STREAM, CODEC_NOT_FOUND,
        ("unable to find codec needed for stream"), (NULL));
        return FALSE;
    }

    /* Configure the element to use the detected engine name and codec, unless
     * they have been using the set_property function.
     */
    if (!imgdec1->engineName) {
        imgdec1->engineName = codec->CE_EngineName;
    }
    if (!imgdec1->codecName) {
        imgdec1->codecName = codec->CE_CodecName;
    }

    if (!caps) {
        GST_INFO("No caps on input.  Using command line values");
        imgdec1->capsSet = TRUE;
        return TRUE;
    }

    capStruct = gst_caps_get_structure(caps, 0);
    mime      = gst_structure_get_name(capStruct);

    GST_INFO("requested sink caps:  %s", gst_caps_to_string(caps));

    /* Generic Video Properties */
    if (!strncmp(mime, "video/", 6)) {
        gint  framerateNum;
        gint  framerateDen;
        gint  width;
        gint  height;
   
        if (!imgdec1->framerateNum) {        
            if (gst_structure_get_fraction(capStruct, "framerate", 
                                &framerateNum, &framerateDen)) {
                imgdec1->framerateNum = framerateNum;
                imgdec1->framerateDen = framerateDen;
            }
        }

        /* Get the width and height of the frame if available */
        if (!imgdec1->width)
            if (gst_structure_get_int(capStruct, "width", &width)) 
                imgdec1->params.maxWidth = width;
        if (!imgdec1->height)
            if (gst_structure_get_int(capStruct, "height", &height))
                imgdec1->params.maxHeight = height;
    }

    /* Shut-down any running image decoder */
    if (!gst_tiimgdec1_exit_image(imgdec1)) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("unable to shut-down running image decoder"), (NULL));
        return FALSE;
    }

    /* Flag that we have run this code at least once */
    imgdec1->capsSet = TRUE;

    GST_LOG("Finish\n");
    return TRUE;
}

/******************************************************************************
 * gst_tiimgdec1_set_sink_caps
 *     Negotiate our sink pad capabilities.
 ******************************************************************************/
static gboolean gst_tiimgdec1_set_sink_caps(GstPad *pad, GstCaps *caps)
{
    GstTIImgdec1 *imgdec1;
    imgdec1   = GST_TIIMGDEC1(gst_pad_get_parent(pad));

    GST_LOG("Begin\n");

    /* If this call fails then unref the gobject */
    if (!gst_tiimgdec1_set_sink_caps_helper(pad, caps)) {
        GST_ELEMENT_ERROR(imgdec1, STREAM, NOT_IMPLEMENTED,
        ("stream type not supported"), (NULL));
        gst_object_unref(imgdec1);
        return FALSE;
    }

    GST_LOG("sink caps negotiation successful\n");
    GST_LOG("Finish\n");
    return TRUE;
}


/******************************************************************************
 * gst_tiimgdec1_set_source_caps
 *     Negotiate our source pad capabilities.
 ******************************************************************************/
static gboolean gst_tiimgdec1_set_source_caps(
                    GstTIImgdec1 *imgdec1, Buffer_Handle hBuf)
{
    BufferGfx_Dimensions  dim;
    GstCaps              *caps;
    gboolean              ret;
    GstPad               *pad;
    guint32              format;

    GST_LOG("Begin\n");
    pad = imgdec1->srcpad;

    /* Create a UYVY caps object using the dimensions from the given buffer */
    BufferGfx_getDimensions(hBuf, &dim);

    format = gst_tiimgdec1_codec_color_space_to_fourcc(imgdec1->params.forceChromaFormat);

    if (format == -1) {
        GST_ERROR("Could not convert codec color space to fourcc");
        return FALSE;
    }
    caps =
        gst_caps_new_simple("video/x-raw-yuv",
            "format",    GST_TYPE_FOURCC,   format,
            "framerate", GST_TYPE_FRACTION, imgdec1->framerateNum,
                                            imgdec1->framerateDen,
            "width",     G_TYPE_INT,        dim.width,
            "height",    G_TYPE_INT,        dim.height,
            NULL);

    /* Set the source pad caps */
    GST_DEBUG("setting source caps to UYVY:  %s", gst_caps_to_string(caps));
    ret = gst_pad_set_caps(pad, caps);
    gst_caps_unref(caps);

    GST_LOG("Finish\n");
    return ret;
}


/******************************************************************************
 * gst_tiimgdec1_sink_event
 *     Perform event processing on the input stream.  At the moment, this
 *     function isn't needed as this element doesn't currently perform any
 *     specialized event processing.  We'll leave it in for now in case we need
 *     it later on.
 ******************************************************************************/
static gboolean gst_tiimgdec1_sink_event(GstPad *pad, GstEvent *event)
{
    GstTIImgdec1 *imgdec1;
    gboolean      ret;

    GST_LOG("Begin\n");

    imgdec1 = GST_TIIMGDEC1(GST_OBJECT_PARENT(pad));

    GST_DEBUG("pad \"%s\" received:  %s\n", GST_PAD_NAME(pad),
        GST_EVENT_TYPE_NAME(event));

    switch (GST_EVENT_TYPE(event)) {

        case GST_EVENT_NEWSEGMENT:
            /* maybe save and/or update the current segment (e.g. for output
             * clipping) or convert the event into one in a different format
             * (e.g. BYTES to TIME) or drop it and set a flag to send a
             * newsegment event in a different format later
             */
            ret = gst_pad_push_event(imgdec1->srcpad, event);
            break;

        case GST_EVENT_EOS:
            /* end-of-stream: process any remaining decoded frame data */
            GST_LOG("no more input; draining remaining decoded image data\n");

            if (!imgdec1->drainingEOS) {
               gst_tiimgdec1_drain_pipeline(imgdec1);
             }

            /* Propagate EOS to downstream elements */
            ret = gst_pad_push_event(imgdec1->srcpad, event);
            break;

        case GST_EVENT_FLUSH_STOP:
            ret = gst_pad_push_event(imgdec1->srcpad, event);
            break;

        /* Unhandled events */
        case GST_EVENT_BUFFERSIZE:
        case GST_EVENT_CUSTOM_BOTH:
        case GST_EVENT_CUSTOM_BOTH_OOB:
        case GST_EVENT_CUSTOM_DOWNSTREAM:
        case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:
        case GST_EVENT_CUSTOM_UPSTREAM:
        case GST_EVENT_FLUSH_START:
        case GST_EVENT_NAVIGATION:
        case GST_EVENT_QOS:
        case GST_EVENT_SEEK:
        case GST_EVENT_TAG:
        default:
            ret = gst_pad_event_default(pad, event);
            break;

    }

    GST_LOG("Finish\n");
    return ret;

}


/******************************************************************************
 * gst_tiimgdec1_chain
 *    This is the main processing routine.  This function receives a buffer
 *    from the sink pad, processes it, and pushes the result to the source
 *    pad.
 ******************************************************************************/
static GstFlowReturn gst_tiimgdec1_chain(GstPad * pad, GstBuffer * buf)
{
    GstTIImgdec1  *imgdec1 = GST_TIIMGDEC1(GST_OBJECT_PARENT(pad));
    GstCaps       *caps    = GST_BUFFER_CAPS(buf);
    GstFlowReturn  flow    = GST_FLOW_OK;
    gboolean       checkResult;

    /* If the decode thread aborted, signal it to let it know it's ok to
     * shut down, and communicate the failure to the pipeline.
     */
    if (gst_tithread_check_status(imgdec1, TIThread_CODEC_ABORTED,
            checkResult)) {
        flow = GST_FLOW_UNEXPECTED;
        goto exit;
    }

    /* If we have not negotiated the caps at least once then do so now */
    if (!imgdec1->capsSet) {
        if (!gst_tiimgdec1_set_sink_caps_helper(pad, caps)) {
            GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
            ("Could not set caps"), (NULL));
            flow = GST_FLOW_UNEXPECTED;
            goto exit;
        }
    }

    /* If our engine handle is currently NULL, then either this is our first
     * buffer or the upstream element has re-negotiated our capabilities which
     * resulted in our engine being closed.  In either case, we need to
     * initialize (or re-initialize) our image decoder to handle the new
     * stream.
     */
    if (imgdec1->hEngine == NULL) {
        if (!gst_tiimgdec1_init_image(imgdec1)) {
            GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
            ("unable to initialize image\n"), (NULL));
            flow = GST_FLOW_UNEXPECTED;
            goto exit;
        }

        GST_TICIRCBUFFER_TIMESTAMP(imgdec1->circBuf) =
            GST_CLOCK_TIME_IS_VALID(GST_BUFFER_TIMESTAMP(buf)) ?
            GST_BUFFER_TIMESTAMP(buf) : 0ULL;
    }

    /* Queue up the encoded data stream into a circular buffer */
    if (!gst_ticircbuffer_queue_data(imgdec1->circBuf, buf)) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, WRITE,
        ("Failed to queue input buffer into circular buffer\n"), (NULL));
        flow = GST_FLOW_UNEXPECTED;
        goto exit;
    }

exit:
    gst_buffer_unref(buf);
    return flow;
}

/*******************************************************************************
 * gst_tiimgdec1_codec_color_space_to_str
 *      Converts the codec color space values to the corresponding
 *      human readable string value
*******************************************************************************/
static char *gst_tiimgdec1_codec_color_space_to_str(int cspace) {
    GST_LOG("Begin");
    switch (cspace) {
        case XDM_YUV_422ILE:
            GST_LOG("Finish");
            return "UYVY";
            break;
        case XDM_YUV_420P:
            GST_LOG("Finish");
            return "YUV420P";
            break;
        case XDM_YUV_422P:
            GST_LOG("Finish");
            return "YUV422P";
            break;
        default:
            GST_ERROR("Unknown xDM color space");
            GST_LOG("Finish");
            return "Unknown";
    }
}

/*******************************************************************************
 * gst_tiimgdec1_codec_color_space_to_fourcc
 *      Converts the codec color space values to the corresponding
 *      fourcc values
*******************************************************************************/
static int gst_tiimgdec1_codec_color_space_to_fourcc(int cspace) {
    GST_LOG("Begin");
    switch (cspace) {
        case XDM_YUV_422ILE:
            GST_LOG("Finish");
            return GST_MAKE_FOURCC('U', 'Y', 'V', 'Y');
            break;
        case XDM_YUV_420P:
            GST_LOG("Finish");
            return GST_MAKE_FOURCC('I', '4', '2', '0');
            break;
        case XDM_YUV_422P:
            GST_LOG("Finish");
            return GST_MAKE_FOURCC('Y', '4', '2', 'B');
            break;
        default:
            GST_ERROR("Unknown xDM color space");
            GST_LOG("Finish");
            return -1;
            break;
    }
}

/*******************************************************************************
 * gst_tiimgdec1_set_codec_attrs
 *     Set the attributes for the decode codec.  The general order is to give
 *     preferdece to values passed in by the user on the command line,
 *     then check the stream for the information.  If the required information
 *     cannot be determined by either method then error out.
 ******************************************************************************/
static gboolean gst_tiimgdec1_set_codec_attrs(GstTIImgdec1 *imgdec1)
{ 
    char *toColor;
    GST_LOG("Begin\n");

    /* Set Resolution 
     *
     * NOTE:  We assume that there is only 1 resolution.  i.e. there is
     *        no seperate value being used for input resolution and
     *        output resolution
     */
    imgdec1->params.maxWidth = 
                    imgdec1->width == 0 ?
                    imgdec1->params.maxWidth :
                    imgdec1->width;
    imgdec1->params.maxHeight =
                    imgdec1->height == 0 ?
                    imgdec1->params.maxHeight :
                    imgdec1->height;
    /* Set displayWidth to 0 in order to use decoded image width */
    imgdec1->dynParams.displayWidth = 0;

    /* Check for valid values (NOTE: minimum width and height are 64) */
    if (imgdec1->params.maxWidth < 64) {
        GST_ERROR("The resolution width (%ld) is too small.  Must be at least 64\n", imgdec1->params.maxWidth);
        return FALSE;
    }
    if (imgdec1->params.maxHeight < 64) {
        GST_ERROR("The resolution height (%ld) is too small.  Must be at least 64\n", imgdec1->params.maxHeight);
        return FALSE;
    }

    /* Make the color human readable */
    toColor = gst_tiimgdec1_codec_color_space_to_str(imgdec1->params.forceChromaFormat);

    GST_DEBUG("\nCodec Parameters:\n"
              "\tparams.maxWidth = %ld\n"
              "\tparams.maxHeight = %ld\n"
              "\tparams.forceChromaFormat = %ld (%s)\n"
              "\tparams.maxScans - %ld\n"
              "\tparams.dataEndianness = %ld\n",
              imgdec1->params.maxWidth, imgdec1->params.maxHeight,
              imgdec1->params.forceChromaFormat, toColor,
              imgdec1->params.maxScans, imgdec1->params.dataEndianness);
    GST_LOG("Finish\n");
    return TRUE;
}

/*******************************************************************************
 * gst_tiimgdec1_codec_color_space_to_dmai
 *      Convert the codec color space type to the corresponding color space
 *      used by DMAI.
*******************************************************************************/
static int gst_tiimgdec1_codec_color_space_to_dmai(int cspace) {
    GST_LOG("Begin");
    switch (cspace) {
        case XDM_YUV_422ILE:
            GST_LOG("Finish");
            return ColorSpace_UYVY;
            break;
        case XDM_YUV_420P:
            GST_LOG("Finish");
            return ColorSpace_YUV420P;
            break;
        case XDM_YUV_422P:
            GST_LOG("Finish");
            return ColorSpace_YUV422P;
            break;
        default:
            GST_ERROR("Unsupported Color Space\n");
            GST_LOG("Finish");
            return -1;
    }
}

/******************************************************************************
 * gst_tiimgdec1_init_image
 *     Initialize or re-initializes the image stream
 ******************************************************************************/
static gboolean gst_tiimgdec1_init_image(GstTIImgdec1 *imgdec1)
{
    Rendezvous_Attrs    rzvAttrs  = Rendezvous_Attrs_DEFAULT;
    struct sched_param  schedParam;
    pthread_attr_t      attr;

    GST_LOG("Begin\n");

    /* If image has already been initialized, shut down previous decoder */
    if (imgdec1->hEngine) {
        if (!gst_tiimgdec1_exit_image(imgdec1)) {
            GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
            ("failed to shut down existing image decoder\n"), (NULL));
            return FALSE;
        }
    }

    /* Make sure we know what codec we're using */
    if (!imgdec1->engineName) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("engine name not specified\n"), (NULL));
        return FALSE;
    }

    if (!imgdec1->codecName) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("codec name not specified\n"), (NULL));
        return FALSE;
    }

    /* Initialize thread status management */
    imgdec1->threadStatus = 0UL;
    pthread_mutex_init(&imgdec1->threadStatusMutex, NULL);

    /* Initialize rendezvous objects for making threads wait on conditions */
    imgdec1->waitOnDecodeThread = Rendezvous_create(2, &rzvAttrs);
    imgdec1->waitOnDecodeDrain  = Rendezvous_create(100, &rzvAttrs);
    imgdec1->drainingEOS        = FALSE;

    /* Initialize custom thread attributes */
    if (pthread_attr_init(&attr)) {
        GST_WARNING("failed to initialize thread attrs\n");
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }

    /* Force the thread to use the system scope */
    if (pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM)) {
        GST_WARNING("failed to set scope attribute\n");
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }

    /* Force the thread to use custom scheduling attributes */
    if (pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED)) {
        GST_WARNING("failed to set schedule inheritance attribute\n");
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }

    /* Set the thread to be fifo real time scheduled */
    if (pthread_attr_setschedpolicy(&attr, SCHED_FIFO)) {
        GST_WARNING("failed to set FIFO scheduling policy\n");
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }

    /* Set the display thread priority */
    schedParam.sched_priority = GstTIVideoThreadPriority;
    if (pthread_attr_setschedparam(&attr, &schedParam)) {
        GST_WARNING("failed to set scheduler parameters\n");
        return FALSE;
    }

    /* Create decoder thread */
    if (pthread_create(&imgdec1->decodeThread, &attr,
            gst_tiimgdec1_decode_thread, (void*)imgdec1)) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("failed to create decode thread\n"), (NULL));
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }
    gst_tithread_set_status(imgdec1, TIThread_CODEC_CREATED);

    /* Destroy the custom thread attributes */
    if (pthread_attr_destroy(&attr)) {
        GST_WARNING("failed to destroy thread attrs\n");
        gst_tiimgdec1_exit_image(imgdec1);
        return FALSE;
    }

    /* Make sure circular buffer and display buffer handles are created by
     * decoder thread.
     */
    Rendezvous_meet(imgdec1->waitOnDecodeThread);

    if (imgdec1->circBuf == NULL || imgdec1->hOutBufTab == NULL) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("decode thread failed to create circbuf or display buffer handles\n"),
        (NULL));
        return FALSE;
    }

    GST_LOG("Finish\n");
    return TRUE;
}


/******************************************************************************
 * gst_tiimgdec1_exit_image
 *    Shut down any running image decoder, and reset the element state.
 ******************************************************************************/
static gboolean gst_tiimgdec1_exit_image(GstTIImgdec1 *imgdec1)
{
    void*    thread_ret;
    gboolean checkResult;

    GST_LOG("Begin\n");

    /* Drain the pipeline if it hasn't already been drained */
    if (!imgdec1->drainingEOS) {
       gst_tiimgdec1_drain_pipeline(imgdec1);
     }

    /* Shut down the decode thread */
    if (gst_tithread_check_status(
            imgdec1, TIThread_CODEC_CREATED, checkResult)) {
        GST_LOG("shutting down decode thread\n");

        Rendezvous_force(imgdec1->waitOnDecodeThread);
        if (pthread_join(imgdec1->decodeThread, &thread_ret) == 0) {
            if (thread_ret == GstTIThreadFailure) {
                GST_DEBUG("decode thread exited with an error condition\n");
            }
        }
    }

    /* Shut down any remaining items */
    if (imgdec1->waitOnDecodeDrain) {
        Rendezvous_delete(imgdec1->waitOnDecodeDrain);
        imgdec1->waitOnDecodeDrain = NULL;
    }

    if (imgdec1->waitOnDecodeThread) {
        Rendezvous_delete(imgdec1->waitOnDecodeThread);
        imgdec1->waitOnDecodeThread = NULL;
    }

    /* Shut down thread status management */
    imgdec1->threadStatus = 0UL;
    pthread_mutex_destroy(&imgdec1->threadStatusMutex);

    GST_LOG("Finish\n");
    return TRUE;
}


/******************************************************************************
 * gst_tiimgdec1_change_state
 *     Manage state changes for the image stream.  The gStreamer documentation
 *     states that state changes must be handled in this manner:
 *        1) Handle ramp-up states
 *        2) Pass state change to base class
 *        3) Handle ramp-down states
 ******************************************************************************/
static GstStateChangeReturn gst_tiimgdec1_change_state(GstElement *element,
                                GstStateChange transition)
{
    GstStateChangeReturn  ret    = GST_STATE_CHANGE_SUCCESS;
    GstTIImgdec1          *imgdec1 = GST_TIIMGDEC1(element);

    GST_LOG("Begin change_state (%d)\n", transition);

    /* Handle ramp-up state changes */
    switch (transition) {
        case GST_STATE_CHANGE_NULL_TO_READY:
            break;
        default:
            break;
    }

    /* Pass state changes to base class */
    ret = GST_ELEMENT_CLASS(parent_class)->change_state(element, transition);
    if (ret == GST_STATE_CHANGE_FAILURE)
        return ret;

    /* Handle ramp-down state changes */
    switch (transition) {
        case GST_STATE_CHANGE_READY_TO_NULL:
            GST_LOG("State changed from READY to NULL.  Shutting down any"
                    "running image decoders\n");
            /* Shut down any running image decoder */
            if (!gst_tiimgdec1_exit_image(imgdec1)) {
                return GST_STATE_CHANGE_FAILURE;
            }
            break;

        default:
            break;
    }

    GST_LOG("Finish change_state\n");
    return ret;
}

/******************************************************************************
 * gst_tiimgdec1_codec_stop
 *     Stop codec engine
 *****************************************************************************/
static gboolean gst_tiimgdec1_codec_stop (GstTIImgdec1  *imgdec1)
{
    /* Shut down remaining items */
    if (imgdec1->circBuf) {
        GstTICircBuffer *circBuf;

        GST_LOG("freeing cicrular input buffer\n");

        circBuf               = imgdec1->circBuf;
        imgdec1->circBuf      = NULL;
        imgdec1->framerateNum = 0;
        imgdec1->framerateDen = 0;
        gst_ticircbuffer_unref(circBuf);
    }

    if (imgdec1->hOutBufTab) {
        GST_LOG("freeing output buffers\n");
        gst_tidmaibuftab_unref(imgdec1->hOutBufTab);
        imgdec1->hOutBufTab = NULL;
    }

    if (imgdec1->hIe) {
        GST_LOG("closing image decoder\n");
        Idec1_delete(imgdec1->hIe);
        imgdec1->hIe = NULL;
    }

    if (imgdec1->hEngine) {
        GST_LOG("closing codec engine\n");
        Engine_close(imgdec1->hEngine);
        imgdec1->hEngine = NULL;
    }

    return TRUE;
}


/******************************************************************************
 * gst_tiimgdec1_codec_start
 *     Initialize codec engine
 *****************************************************************************/
static gboolean gst_tiimgdec1_codec_start (GstTIImgdec1  *imgdec1)
{
    BufferGfx_Attrs        gfxAttrs  = BufferGfx_Attrs_DEFAULT;

    /* Open the codec engine */
    GST_LOG("opening codec engine \"%s\"\n", imgdec1->engineName);
    imgdec1->hEngine = Engine_open((Char *) imgdec1->engineName, NULL, NULL);

    if (imgdec1->hEngine == NULL) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("failed to open codec engine \"%s\"\n", imgdec1->engineName), (NULL));
        return FALSE;
    }
    GST_LOG("opened codec engine \"%s\" successfully\n", imgdec1->engineName);

    if (!gst_tiimgdec1_set_codec_attrs(imgdec1)) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("Error while trying to set the codec attrs\n"), (NULL));
        return FALSE;
    }

    GST_LOG("opening image decoder \"%s\"\n", imgdec1->codecName);
    imgdec1->hIe = Idec1_create(imgdec1->hEngine, (Char*)imgdec1->codecName,
                      &imgdec1->params, &imgdec1->dynParams);

    if (imgdec1->hIe == NULL) {
        GST_ELEMENT_ERROR(imgdec1, STREAM, CODEC_NOT_FOUND,
        ("failed to create image decoder: %s\n", imgdec1->codecName), (NULL));
        GST_DEBUG("Verify that the values being used for input and output ColorSpace are supported by your codec.\n");
        GST_LOG("closing codec engine\n");
        return FALSE;
    }

    /* Create a circular input buffer */
    imgdec1->circBuf = gst_ticircbuffer_new(
                           Idec1_getInBufSize(imgdec1->hIe), 3, FALSE);

    if (imgdec1->circBuf == NULL) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, NO_SPACE_LEFT,
        ("failed to create circular input buffer\n"), (NULL));
        return FALSE;
    }

    /* Display buffer contents if displayBuffer=TRUE was specified */
    gst_ticircbuffer_set_display(imgdec1->circBuf, imgdec1->displayBuffer);

    /* Define the number of display buffers to allocate.  This number must be
     * at least 1, but should be more if codecs don't return a display buffer
     * after every process call.  If this has not been set via set_property(),
     * default to the value set above based on device type.
     */
    if (imgdec1->numOutputBufs == 0) {
        imgdec1->numOutputBufs = 1;
    }

    /* Create codec output buffers */
    GST_LOG("creating output buffer table\n");
    gfxAttrs.colorSpace     = gst_tiimgdec1_codec_color_space_to_dmai(imgdec1->params.forceChromaFormat);
    gfxAttrs.dim.width      = imgdec1->params.maxWidth;
    gfxAttrs.dim.height     = imgdec1->params.maxHeight;
    gfxAttrs.dim.lineLength = BufferGfx_calcLineLength(
                                  gfxAttrs.dim.width, gfxAttrs.colorSpace);

    gfxAttrs.bAttrs.memParams.align = 128;

    /* By default, new buffers are marked as in-use by the codec */
    gfxAttrs.bAttrs.useMask = gst_tidmaibuffer_CODEC_FREE;

    imgdec1->hOutBufTab = gst_tidmaibuftab_new(imgdec1->numOutputBufs,
        Idec1_getOutBufSize(imgdec1->hIe),
        BufferGfx_getBufferAttrs(&gfxAttrs));

    if (imgdec1->hOutBufTab == NULL) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, NO_SPACE_LEFT,
        ("failed to create output buffers\n"), (NULL));
        return FALSE;
    }

    return TRUE;
}


/******************************************************************************
 * gst_tiimgdec1_decode_thread
 *     Call the image codec to process a full input buffer
 ******************************************************************************/
static void* gst_tiimgdec1_decode_thread(void *arg)
{
    GstTIImgdec1           *imgdec1        = GST_TIIMGDEC1(gst_object_ref(arg));
    GstBuffer              *encDataWindow  = NULL;
    gboolean               codecFlushed    = FALSE;
    void                   *threadRet      = GstTIThreadSuccess;
    BufferGfx_Attrs        gfxAttrs        = BufferGfx_Attrs_DEFAULT;
    Buffer_Handle          hDstBuf;
    Int32                  encDataConsumed;
    GstClockTime           encDataTime;
    GstClockTime           frameDuration;
    Buffer_Handle          hEncDataWindow;
    BufferGfx_Dimensions   dim;
    GstBuffer              *outBuf;
    Int                    bufIdx;
    Int                    ret;


    GST_LOG("Begin\n");
    /* Calculate the duration of a single frame in this stream */
    frameDuration = gst_tiimgdec1_frame_duration(imgdec1);

    /* Initialize codec engine */
    ret = gst_tiimgdec1_codec_start(imgdec1);

    /* Notify main thread that is ok to continue initialization */
    Rendezvous_meet(imgdec1->waitOnDecodeThread);
    Rendezvous_reset(imgdec1->waitOnDecodeThread);

    if (ret == FALSE) {
        GST_ELEMENT_ERROR(imgdec1, RESOURCE, FAILED,
        ("failed to start codec\n"), (NULL));
        goto thread_exit;
    }

    /* Main thread loop */
    while (TRUE) {

        /* Obtain an encoded data frame */
        encDataWindow  = gst_ticircbuffer_get_data(imgdec1->circBuf);
        encDataTime    = GST_BUFFER_TIMESTAMP(encDataWindow);
        hEncDataWindow = GST_TIDMAIBUFFERTRANSPORT_DMAIBUF(encDataWindow);

        /* If we received a data frame of zero size, there is no more data to
         * process -- exit the thread.  If we weren't told that we are
         * draining the pipeline, something is not right, so exit with an
         * error.
         */
        if (GST_BUFFER_SIZE(encDataWindow) == 0) {
            GST_LOG("no image data remains\n");
            if (!imgdec1->drainingEOS) {
                goto thread_failure;
            }

            goto thread_exit;
        }

        /* Obtain a free output buffer for the decoded data */
        if (!(hDstBuf = gst_tidmaibuftab_get_buf(imgdec1->hOutBufTab))) {
            GST_ELEMENT_ERROR(imgdec1, RESOURCE, READ,
                ("failed to get a free contiguous buffer from BufTab\n"), 
                (NULL));
            goto thread_failure;
        }

        /* Make sure the whole buffer is used for output */
        BufferGfx_resetDimensions(hDstBuf);

        /* Create a BufferGfx object that will point to a reference buffer from
         * The circular buffer.  This is needed for the decoder which requires
         * that the input buffer be a BufferGfx object.
         */
        BufferGfx_getBufferAttrs(&gfxAttrs)->reference = TRUE;
        imgdec1->hInBuf = Buffer_create(Idec1_getInBufSize(imgdec1->hIe),
                                BufferGfx_getBufferAttrs(&gfxAttrs));
        Buffer_setUserPtr(imgdec1->hInBuf, Buffer_getUserPtr(hEncDataWindow));
        Buffer_setNumBytesUsed(imgdec1->hInBuf, 
                               Buffer_getSize(hEncDataWindow));

        /* Set the dimensions of the buffer to the resolution */
        BufferGfx_getDimensions(hDstBuf, &dim);
        dim.width = imgdec1->params.maxWidth;
        dim.height = imgdec1->params.maxHeight;
        BufferGfx_setColorSpace(imgdec1->hInBuf,
                     ColorSpace_UYVY); 
        dim.lineLength = BufferGfx_calcLineLength(imgdec1->params.maxWidth,
                                  gfxAttrs.colorSpace);

        BufferGfx_setDimensions(hDstBuf, &dim);
        Buffer_setNumBytesUsed(hDstBuf, Buffer_getSize(hDstBuf));

        /* Invoke the image decoder */
        GST_LOG("invoking the image decoder\n");
        ret             = Idec1_process(imgdec1->hIe, imgdec1->hInBuf, hDstBuf);
        encDataConsumed = (codecFlushed) ? 0 :
                          Buffer_getNumBytesUsed(hEncDataWindow);

        /* Delete the temporary Graphics buffer used for the decode process */
        Buffer_delete(imgdec1->hInBuf);

        if (ret < 0) {
            GST_ELEMENT_ERROR(imgdec1, STREAM, DECODE, 
            ("failed to decode image buffer\n"), (NULL));
            goto thread_failure;
        }

        /* If no data was used we must have some kind of decode error */
        if (ret == Dmai_EBITERROR && encDataConsumed == 0 && !codecFlushed) {
            GST_ELEMENT_ERROR(imgdec1, STREAM, DECODE,
            ("fatal bit error\n"), (NULL));
            goto thread_failure;
        }

        if (ret > 0) {
            GST_LOG("Idec1_process returned success code %d\n", ret); 
        }

        /* Release the referdece buffer, and tell the circular buffer how much
         * data was consumed.
         */
        ret = gst_ticircbuffer_data_consumed(imgdec1->circBuf, encDataWindow,
                  encDataConsumed);
        encDataWindow = NULL;

        if (!ret) {
            goto thread_failure;
        }


        /* Set the source pad capabilities based on the decoded frame
         * properties.
         */
        gst_tiimgdec1_set_source_caps(imgdec1, hDstBuf);

        /* Create a DMAI transport buffer object to carry a DMAI buffer to
         * the source pad.  The transport buffer knows how to release the
         * buffer for re-use in this element when the source pad calls
         * gst_buffer_unref().
         */
        outBuf = gst_tidmaibuffertransport_new(hDstBuf, imgdec1->hOutBufTab);
        gst_buffer_set_data(outBuf, GST_BUFFER_DATA(outBuf),
            gst_ti_correct_display_bufSize(hDstBuf));
        gst_buffer_set_caps(outBuf, GST_PAD_CAPS(imgdec1->srcpad));

        /* If we have a valid time stamp, set it on the buffer */
        if (imgdec1->genTimeStamps &&
            GST_CLOCK_TIME_IS_VALID(encDataTime)) {
            GST_LOG("image timestamp value: %llu\n", encDataTime);
            GST_BUFFER_TIMESTAMP(outBuf) = encDataTime;
            GST_BUFFER_DURATION(outBuf)  = frameDuration;
        }
        else {
            GST_BUFFER_TIMESTAMP(outBuf) = GST_CLOCK_TIME_NONE;
        }

        /* Tell circular buffer how much time we consumed */
        gst_ticircbuffer_time_consumed(imgdec1->circBuf, frameDuration);

        /* Push the transport buffer to the source pad */
        GST_LOG("pushing display buffer to source pad\n");

        if (gst_pad_push(imgdec1->srcpad, outBuf) != GST_FLOW_OK) {
            GST_DEBUG("push to source pad failed\n");
            goto thread_failure;
        }

        /* Release buffers no longer in use by the codec */
        Buffer_freeUseMask(hDstBuf, gst_tidmaibuffer_CODEC_FREE);
    }

thread_failure:

    gst_tithread_set_status(imgdec1, TIThread_CODEC_ABORTED);
    gst_ticircbuffer_consumer_aborted(imgdec1->circBuf);
    threadRet = GstTIThreadFailure;

thread_exit:
 
    /* Re-claim any buffers owned by the codec */
    bufIdx = BufTab_getNumBufs(GST_TIDMAIBUFTAB_BUFTAB(imgdec1->hOutBufTab));

    while (bufIdx-- > 0) {
        Buffer_Handle hBuf = BufTab_getBuf(
            GST_TIDMAIBUFTAB_BUFTAB(imgdec1->hOutBufTab), bufIdx);
        Buffer_freeUseMask(hBuf, gst_tidmaibuffer_CODEC_FREE);
    }

    /* Release the last buffer we retrieved from the circular buffer */
    if (encDataWindow) {
        gst_ticircbuffer_data_consumed(imgdec1->circBuf, encDataWindow, 0);
    }

    /* We have to wait to shut down this thread until we can guarantee that
     * no more input buffers will be queued into the circular buffer
     * (we're about to delete it).  
     */
    Rendezvous_meet(imgdec1->waitOnDecodeThread);
    Rendezvous_reset(imgdec1->waitOnDecodeThread);

    /* Notify main thread that we are done draining before we shutdown the
     * codec, or we will hang.  We proceed in this order so the EOS event gets
     * propagated downstream before we attempt to shut down the codec.  The
     * codec-shutdown process will block until all BufTab buffers have been
     * released, and downstream-elements may hang on to buffers until
     * they get the EOS.
     */
    Rendezvous_force(imgdec1->waitOnDecodeDrain);

    /* Stop codec engine */
    if (gst_tiimgdec1_codec_stop(imgdec1) < 0) {
        GST_ERROR("failed to stop codec\n");
    }

    gst_object_unref(imgdec1);

    GST_LOG("Finish image decode_thread (%d)\n", (int)threadRet);
    return threadRet;
}


/******************************************************************************
 * gst_tiimgdec1_drain_pipeline
 *    Wait for the decode thread to finish processing queued input data.
 ******************************************************************************/
static void gst_tiimgdec1_drain_pipeline(GstTIImgdec1 *imgdec1)
{
    gboolean checkResult;

    /* If the decode thread hasn't been created, there is nothing to drain. */
    if (!gst_tithread_check_status(
             imgdec1, TIThread_CODEC_CREATED, checkResult)) {
        return;
    }

    imgdec1->drainingEOS = TRUE;
    gst_ticircbuffer_drain(imgdec1->circBuf, TRUE);

    /* Tell the decode thread that it is ok to shut down */
    Rendezvous_force(imgdec1->waitOnDecodeThread);

    /* Wait for the decoder to finish draining */
    Rendezvous_meet(imgdec1->waitOnDecodeDrain);
}


/******************************************************************************
 * gst_tiimgdec1_frame_duration
 *    Return the duration of a single frame in nanoseconds.
 ******************************************************************************/
static GstClockTime gst_tiimgdec1_frame_duration(GstTIImgdec1 *imgdec1)
{
    GST_LOG("Begin\n");
    /* Default to 29.97 if the frame rate was not specified */
    if (imgdec1->framerateNum == 0 && imgdec1->framerateDen == 0) {
        GST_DEBUG("framerate not specified; using 29.97fps");
        imgdec1->framerateNum = 30000;
        imgdec1->framerateDen = 1001;
    }

    GST_LOG("Finish\n");
    return (GstClockTime)
        ((1 / ((gdouble)imgdec1->framerateNum/(gdouble)imgdec1->framerateDen))
         * GST_SECOND);
}


/******************************************************************************
 * Custom ViM Settings for editing this file
 ******************************************************************************/
#if 0
 Tabs (use 4 spaces for indentation)
 vim:set tabstop=4:      /* Use 4 spaces for tabs          */
 vim:set shiftwidth=4:   /* Use 4 spaces for >> operations */
 vim:set expandtab:      /* Expand tabs into white spaces  */
#endif
